set RandomSeed 123

#
# Official Delphes card prepared by FCC-hh collaboration
#
#  Main authors:  Michele Selvaggi (CERN)
#
#  Released on: July 27th, 2022
#
#
#  Configuration: FCC-hh Scenario II (nominal revisited)
#
#
#   - removed DenseTrackFilter (not relevant only for pT> 5 TeV jet studies)
#   - nominal CLD detector tracking efficiency
#       -  https://indico.cern.ch/event/1064327/contributions/4893180/attachments/2452299/4204992/220601_FCCWeek_CLD_sailer.pdf
#   - nominal FCChh detector tracking/muon resolution for scenario I
#   - use DualRedoutCalorimeter module for simplicity:
#      - ECAL is CMS crystals (test beam)
#      - HCAL is DualReadout
#   - BTagging (3WPs) shape based on latest ParticleNet Run II (still private):
#      - nominal PNet will be scenario II, for scenario I(III) improve(degrade) btagging eff by +/-5%
#   - TauTagging (3WPs) based on DeepJet CMS:
#      - https://arxiv.org/pdf/2201.08458.pdf
#
#   TODO:
#
#
#   - optimise PF parameters using mbb and maa and eventually on boosted jets
#   - check that PF parameters optimisation does not affect electron isolation at high pT
#   - check if can propagate correctly BTag WPs in FCCSW
#   - FatJet tagging?
#
#######################################
# Order of execution of various modules
#######################################

set ExecutionPath {

  ParticlePropagator

  ChargedHadronTrackingEfficiency
  ElectronTrackingEfficiency
  MuonTrackingEfficiency

  TrackMergerPre
  TrackSmearing
  TrackMerger

  ForwardLooperTracks

  Calorimeter
  EFlowTrackMerger
  EFlowMerger
  EFlowFilter

  PhotonEfficiency
  PhotonIsolation

  ElectronFilter
  ElectronEfficiency
  ElectronIsolation

  MuonFilter
  MuonEfficiency
  MuonIsolation

  NeutrinoFilter
  MissingET
  GenMissingET

  GenJetFinder02
  GenJetFinder04
  GenJetFinder08
  GenJetFinder15

  FastJetFinder02
  FastJetFinder04
  FastJetFinder08
  FastJetFinder15

  TowerMerger
  CaloJetFinder02
  CaloJetFinder04
  CaloJetFinder08
  CaloJetFinder15

  TrackJetFinder02
  TrackJetFinder04
  TrackJetFinder08
  TrackJetFinder15

  JetEnergyScale
  JetFlavorAssociation

  BTaggingL
  BTaggingM
  BTaggingT

  CTaggingL
  CTaggingM
  CTaggingT

  TauTaggingL
  TauTaggingM
  TauTaggingT

  ScalarHT

  UniqueObjectFinder

  TreeWriter
}

#####################################
# Track propagation to calorimeters
#####################################

module ParticlePropagator ParticlePropagator {
  set InputArray Delphes/stableParticles
  set OutputArray stableParticles
  set ChargedHadronOutputArray chargedHadrons
  set ElectronOutputArray electrons
  set MuonOutputArray muons

  # radius of the magnetic field coverage, in m
  set Radius 1.5
  # half-length of the magnetic field coverage, in m
  set HalfLength 5

  # magnetic field
  set Bz 4.0
}



####################################
# Charged hadron tracking efficiency
####################################

module Efficiency ChargedHadronTrackingEfficiency {
  set InputArray ParticlePropagator/chargedHadrons
  set OutputArray chargedHadrons
  set UseMomentumVector true

 # TBC (which eta_max ? which pT min?)

 # tracking efficiency formula for charged hadrons

  set EfficiencyFormula {

  0.95 * (
  (pt <= 0.1) * (0.00) +

  (abs(eta) <= 2.5) * (pt > 0.1 && pt <= 0.2) * (0.93) +
  (abs(eta) <= 2.5) * (pt > 0.2 && pt <= 0.3) * (0.95) +
  (abs(eta) <= 2.5) * (pt > 0.3 && pt <= 0.5) * (0.97) +
  (abs(eta) <= 2.5) * (pt > 0.5 && pt <= 1) * (0.99) +
  (abs(eta) <= 2.5) * (pt > 1) * (1.00) +

  0.95 * (
    (abs(eta) > 2.5 && abs(eta) <= 4) * (pt > 0.1 && pt <= 0.2) * (0.93) +
    (abs(eta) > 2.5 && abs(eta) <= 4) * (pt > 0.2 && pt <= 0.3) * (0.95) +
    (abs(eta) > 2.5 && abs(eta) <= 4) * (pt > 0.3 && pt <= 0.5) * (0.97) +
    (abs(eta) > 2.5 && abs(eta) <= 4) * (pt > 0.5 && pt <= 1) * (0.99) +
    (abs(eta) > 2.5 && abs(eta) <= 4) * (pt > 1) * (1.00)
  )+
  0.90 * (
    (abs(eta) > 4.0 && abs(eta) <= 6.0) * (pt > 0.1 && pt <= 0.2) * (0.93) +
    (abs(eta) > 4.0 && abs(eta) <= 6.0) * (pt > 0.2 && pt <= 0.3) * (0.95) +
    (abs(eta) > 4.0 && abs(eta) <= 6.0) * (pt > 0.3 && pt <= 0.5) * (0.97) +
    (abs(eta) > 4.0 && abs(eta) <= 6.0) * (pt > 0.5 && pt <= 1) * (0.99) +
    (abs(eta) > 4.0 && abs(eta) <= 6.0) * (pt > 1) * (1.00)
  )+
  (abs(eta) > 6.0) * (0.00)
  )
  }

}

##############################
# Electron tracking efficiency
##############################

module Efficiency ElectronTrackingEfficiency {
  set InputArray ParticlePropagator/electrons
  set OutputArray electrons
  set UseMomentumVector true

  set EfficiencyFormula {
  0.95 * (
  (pt <= 0.1) * (0.00) +

  (abs(eta) <= 2.5) * (pt > 0.1 && pt <= 0.2) * (0.93) +
  (abs(eta) <= 2.5) * (pt > 0.2 && pt <= 0.3) * (0.95) +
  (abs(eta) <= 2.5) * (pt > 0.3 && pt <= 0.5) * (0.97) +
  (abs(eta) <= 2.5) * (pt > 0.5 && pt <= 1) * (0.99) +
  (abs(eta) <= 2.5) * (pt > 1) * (1.00) +

  0.95 * (
    (abs(eta) > 2.5 && abs(eta) <= 4) * (pt > 0.1 && pt <= 0.2) * (0.93) +
    (abs(eta) > 2.5 && abs(eta) <= 4) * (pt > 0.2 && pt <= 0.3) * (0.95) +
    (abs(eta) > 2.5 && abs(eta) <= 4) * (pt > 0.3 && pt <= 0.5) * (0.97) +
    (abs(eta) > 2.5 && abs(eta) <= 4) * (pt > 0.5 && pt <= 1) * (0.99) +
    (abs(eta) > 2.5 && abs(eta) <= 4) * (pt > 1) * (1.00)
  )+
  0.90 * (
    (abs(eta) > 4.0 && abs(eta) <= 6.0) * (pt > 0.1 && pt <= 0.2) * (0.93) +
    (abs(eta) > 4.0 && abs(eta) <= 6.0) * (pt > 0.2 && pt <= 0.3) * (0.95) +
    (abs(eta) > 4.0 && abs(eta) <= 6.0) * (pt > 0.3 && pt <= 0.5) * (0.97) +
    (abs(eta) > 4.0 && abs(eta) <= 6.0) * (pt > 0.5 && pt <= 1) * (0.99) +
    (abs(eta) > 4.0 && abs(eta) <= 6.0) * (pt > 1) * (1.00)
  )+
  (abs(eta) > 6.0) * (0.00)
  )
  }


}

##########################
# Muon tracking efficiency
##########################

module Efficiency MuonTrackingEfficiency {
  set InputArray ParticlePropagator/muons
  set OutputArray muons
  set UseMomentumVector true

  set EfficiencyFormula {
  0.98*(
  (pt <= 0.1) * (0.00) +

  (abs(eta) <= 2.5) * (pt > 0.1 && pt <= 0.2) * (0.99) +
  (abs(eta) <= 2.5) * (pt > 0.2 && pt <= 0.3) * (0.99) +
  (abs(eta) <= 2.5) * (pt > 0.3 && pt <= 0.5) * (0.99) +
  (abs(eta) <= 2.5) * (pt > 0.5 && pt <= 1) * (0.99) +
  (abs(eta) <= 2.5) * (pt > 1) * (1.00) +

  0.975 * (
    (abs(eta) > 2.5 && abs(eta) <= 4) * (pt > 0.1 && pt <= 0.2) * (0.99) +
    (abs(eta) > 2.5 && abs(eta) <= 4) * (pt > 0.2 && pt <= 0.3) * (0.99) +
    (abs(eta) > 2.5 && abs(eta) <= 4) * (pt > 0.3 && pt <= 0.5) * (0.99) +
    (abs(eta) > 2.5 && abs(eta) <= 4) * (pt > 0.5 && pt <= 1) * (0.99) +
    (abs(eta) > 2.5 && abs(eta) <= 4) * (pt > 1) * (1.00)
  )+
  0.95 * (
    (abs(eta) > 4.0 && abs(eta) <= 6.0) * (pt > 0.1 && pt <= 0.2) * (0.99) +
    (abs(eta) > 4.0 && abs(eta) <= 6.0) * (pt > 0.2 && pt <= 0.3) * (0.99) +
    (abs(eta) > 4.0 && abs(eta) <= 6.0) * (pt > 0.3 && pt <= 0.5) * (0.99) +
    (abs(eta) > 4.0 && abs(eta) <= 6.0) * (pt > 0.5 && pt <= 1) * (0.99) +
    (abs(eta) > 4.0 && abs(eta) <= 6.0) * (pt > 1) * (1.00)
  )+
  (abs(eta) > 6.0) * (0.00)
  )
  }

}

##############
# Track merger
##############

module Merger TrackMergerPre {
# add InputArray InputArray
  add InputArray ChargedHadronTrackingEfficiency/chargedHadrons
  add InputArray ElectronTrackingEfficiency/electrons
  add InputArray MuonTrackingEfficiency/muons
  set OutputArray tracks
}

########################################
# Smearing for charged tracks
########################################

module TrackCovariance TrackSmearing {

    set InputArray TrackMergerPre/tracks
    set OutputArray tracks

    ## minimum number of hits to accept a track
    set NMinHits 8

    ## magnetic field
    set Bz 4.0

    ## scale factors
    # set ElectronScaleFactor  {1.25}


    set DetectorGeometry {

      # Layer type 1 = R (barrel) or 2 = z (forward/backward)
      # Layer label
      # Minimum dimension z for barrel or R for forward
      # Maximum dimension z for barrel or R for forward
      # R/z location of layer
      # Thickness (meters)
      # Radiation length (meters)
      # Number of measurements in layers (1D or 2D)
      # Stereo angle (rad) - 0(pi/2) = axial(z) layer - Upper side
      # Stereo angle (rad) - 0(pi/2) = axial(z) layer - Lower side
      # Resolution Upper side (meters) - 0 = no measurement
      # Resolution Lower side (meters) - 0 = no measurement
      # measurement flag = T, scattering only = F
      # Barrel/endcap	Name	Z min (m)	Z max (m)	R (m)	w (m)	X0	n_meas	th_up	th_down	reso_up	reso_down	flag
      1 PIPE -18 18 0.02 8E-4 0.35276 0 0 0 0 0 0

      1	INB	-0.685	0.685	0.025	0.000937	0.0937	2	0	1.5708	7.50E-06	1.50E-05	1
      1	INB	-0.82	0.82	0.06	0.000937	0.0937	2	0	1.5708	7.50E-06	1.50E-05	1
      1	INB	-0.82	0.82	0.1	0.000937	0.0937	2	0	1.5708	7.50E-06	1.50E-05	1
      1	INB	-0.82	0.82	0.15	0.000937	0.0937	2	0	1.5708	7.50E-06	1.50E-05	1
      1	INB	-0.82	0.82	0.27	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      1	INB	-0.82	0.82	0.4	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1

      1	OUTB	-2.25	2.25	0.53	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      1	OUTB	-2.25	2.25	0.7424	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      1	OUTB	-2.25	2.25	0.93724	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.014434	1
      1	OUTB	-2.25	2.25	1.13201	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.014434	1
      1	OUTB	-2.25	2.25	1.32673	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.014434	1
      1	OUTB	-2.25	2.25	1.53955	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.014434	1

      # Barrel/endcap	Name	R min (m)	R max (m)	Z (m)	w (m)	X0	n_meas	th_up	th_down	reso_up	reso_down	flag
      2	INC1R1	0.025	0.1047	0.95	0.0014055	0.0937	2	0	1.5708	7.50E-06	0.000015	1
      2	INC1R2	0.1013	0.2046	0.95	0.0014055	0.0937	2	0	1.5708	9.50E-06	0.00003	1
      2	INC1R3	0.1989	0.3034	0.95	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	INC1R4	0.3026	0.404	0.95	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
                              
      2	INC1R1	0.025	0.1047	-0.95	0.0014055	0.0937	2	0	1.5708	0.0000075	0.000015	1
      2	INC1R2	0.1013	0.2046	-0.95	0.0014055	0.0937	2	0	1.5708	0.0000095	0.00003	1
      2	INC1R3	0.1989	0.3034	-0.95	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	INC1R4	0.3026	0.404	-0.95	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1

      2	INC2R1	0.025	0.1047	1.1785	0.0014055	0.0937	2	0	1.5708	7.50E-06	0.000015	1
      2	INC2R2	0.1013	0.2046	1.1785	0.0014055	0.0937	2	0	1.5708	9.50E-06	0.00003	1
      2	INC2R3	0.1989	0.3034	1.1785	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	INC2R4	0.3026	0.404	1.1785	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
                              
      2	INC2R1	0.025	0.1047	-1.1785	0.0014055	0.0937	2	0	1.5708	0.0000075	0.000015	1
      2	INC2R2	0.1013	0.2046	-1.1785	0.0014055	0.0937	2	0	1.5708	0.0000095	0.00003	1
      2	INC2R3	0.1989	0.3034	-1.1785	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	INC2R4	0.3026	0.404	-1.1785	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1

      2	INC3R1	0.025	0.1047	1.462	0.0014055	0.0937	2	0	1.5708	7.50E-06	0.000015	1
      2	INC3R2	0.1013	0.2046	1.462	0.0014055	0.0937	2	0	1.5708	9.50E-06	0.00003	1
      2	INC3R3	0.1989	0.3034	1.462	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	INC3R4	0.3026	0.404	1.462	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
                              
      2	INC3R1	0.025	0.1047	-1.462	0.0014055	0.0937	2	0	1.5708	0.0000075	0.000015	1
      2	INC3R2	0.1013	0.2046	-1.462	0.0014055	0.0937	2	0	1.5708	0.0000095	0.00003	1
      2	INC3R3	0.1989	0.3034	-1.462	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	INC3R4	0.3026	0.404	-1.462	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1

      2	INC4R1	0.025	0.1047	1.8137	0.0014055	0.0937	2	0	1.5708	7.50E-06	0.000015	1
      2	INC4R2	0.1013	0.2046	1.8137	0.0014055	0.0937	2	0	1.5708	9.50E-06	0.00003	1
      2	INC4R3	0.1989	0.3034	1.8137	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	INC4R4	0.3026	0.404	1.8137	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
                              
      2	INC4R1	0.025	0.1047	-1.8137	0.0014055	0.0937	2	0	1.5708	0.0000075	0.000015	1
      2	INC4R2	0.1013	0.2046	-1.8137	0.0014055	0.0937	2	0	1.5708	0.0000095	0.00003	1
      2	INC4R3	0.1989	0.3034	-1.8137	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	INC4R4	0.3026	0.404	-1.8137	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1

      2	INC5R1	0.025	0.1047	2.25	0.0014055	0.0937	2	0	1.5708	7.50E-06	0.000015	1
      2	INC5R2	0.1013	0.2046	2.25	0.0014055	0.0937	2	0	1.5708	9.50E-06	0.00003	1
      2	INC5R3	0.1989	0.3034	2.25	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	INC5R4	0.3026	0.404	2.25	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
                              
      2	INC5R1	0.025	0.1047	-2.25	0.0014055	0.0937	2	0	1.5708	0.0000075	0.000015	1
      2	INC5R2	0.1013	0.2046	-2.25	0.0014055	0.0937	2	0	1.5708	0.0000095	0.00003	1
      2	INC5R3	0.1989	0.3034	-2.25	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	INC5R4	0.3026	0.404	-2.25	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1

      2	OUTD1R1	0.025	0.1042	2.625	0.0014055	0.0937	2	0	1.5708	7.50E-06	0.000015	1
      2	OUTD1R2	0.0999	0.2031	2.625	0.0014055	0.0937	2	0	1.5708	9.50E-06	0.00003	1
      2	OUTD1R3	0.1988	0.3031	2.625	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD1R4	0.3014	0.4026	2.625	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD1R5	0.3987	0.5044	2.625	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD1R6	0.5035	0.6036	2.625	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD1R7	0.599	0.7032	2.625	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD1R8	0.7016	0.8056	2.625	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD1R9	0.7989	0.9027	2.625	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD1R10	0.9017	1.0054	2.625	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD1R11	0.9979	1.1015	2.625	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD1R12	1.1009	1.2044	2.625	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD1R13	1.1959	1.2993	2.625	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD1R14	1.2991	1.4024	2.625	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD1R15	1.393	1.4963	2.625	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD1R16	1.4964	1.5493	2.625	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
                              
      2	OUTD1R1	0.025	0.1042	-2.625	0.0014055	0.0937	2	0	1.5708	0.0000075	0.000015	1
      2	OUTD1R2	0.0999	0.2031	-2.625	0.0014055	0.0937	2	0	1.5708	0.0000095	0.00003	1
      2	OUTD1R3	0.1988	0.3031	-2.625	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD1R4	0.3014	0.4026	-2.625	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD1R5	0.3987	0.5044	-2.625	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD1R6	0.5035	0.6036	-2.625	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD1R7	0.599	0.7032	-2.625	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD1R8	0.7016	0.8056	-2.625	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD1R9	0.7989	0.9027	-2.625	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD1R10	0.9017	1.0054	-2.625	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD1R11	0.9979	1.1015	-2.625	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD1R12	1.1009	1.2044	-2.625	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD1R13	1.1959	1.2993	-2.625	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD1R14	1.2991	1.4024	-2.625	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD1R15	1.393	1.4963	-2.625	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD1R16	1.4964	1.5493	-2.625	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1

      2	OUTD2R1	0.025	0.1042	2.9861	0.0014055	0.0937	2	0	1.5708	7.50E-06	0.000015	1
      2	OUTD2R2	0.0999	0.2031	2.9861	0.0014055	0.0937	2	0	1.5708	9.50E-06	0.00003	1
      2	OUTD2R3	0.1988	0.3031	2.9861	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD2R4	0.3014	0.4026	2.9861	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD2R5	0.3987	0.5044	2.9861	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD2R6	0.5035	0.6036	2.9861	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD2R7	0.599	0.7032	2.9861	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD2R8	0.7016	0.8056	2.9861	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD2R9	0.7989	0.9027	2.9861	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD2R10	0.9017	1.0054	2.9861	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD2R11	0.9979	1.1015	2.9861	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD2R12	1.1009	1.2044	2.9861	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD2R13	1.1959	1.2993	2.9861	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD2R14	1.2991	1.4024	2.9861	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD2R15	1.393	1.4963	2.9861	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD2R16	1.4964	1.5493	2.9861	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
                              
      2	OUTD2R1	0.025	0.1042	-2.9861	0.0014055	0.0937	2	0	1.5708	0.0000075	0.000015	1
      2	OUTD2R2	0.0999	0.2031	-2.9861	0.0014055	0.0937	2	0	1.5708	0.0000095	0.00003	1
      2	OUTD2R3	0.1988	0.3031	-2.9861	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD2R4	0.3014	0.4026	-2.9861	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD2R5	0.3987	0.5044	-2.9861	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD2R6	0.5035	0.6036	-2.9861	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD2R7	0.599	0.7032	-2.9861	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD2R8	0.7016	0.8056	-2.9861	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD2R9	0.7989	0.9027	-2.9861	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD2R10	0.9017	1.0054	-2.9861	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD2R11	0.9979	1.1015	-2.9861	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD2R12	1.1009	1.2044	-2.9861	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD2R13	1.1959	1.2993	-2.9861	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD2R14	1.2991	1.4024	-2.9861	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD2R15	1.393	1.4963	-2.9861	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD2R16	1.4964	1.5493	-2.9861	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1

      2	OUTD3R1	0.025	0.1042	3.3968	0.0014055	0.0937	2	0	1.5708	7.50E-06	0.000015	1
      2	OUTD3R2	0.0999	0.2031	3.3968	0.0014055	0.0937	2	0	1.5708	9.50E-06	0.00003	1
      2	OUTD3R3	0.1988	0.3031	3.3968	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD3R4	0.3014	0.4026	3.3968	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD3R5	0.3987	0.5044	3.3968	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD3R6	0.5035	0.6036	3.3968	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD3R7	0.599	0.7032	3.3968	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD3R8	0.7016	0.8056	3.3968	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD3R9	0.7989	0.9027	3.3968	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD3R10	0.9017	1.0054	3.3968	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD3R11	0.9979	1.1015	3.3968	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD3R12	1.1009	1.2044	3.3968	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD3R13	1.1959	1.2993	3.3968	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD3R14	1.2991	1.4024	3.3968	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD3R15	1.393	1.4963	3.3968	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD3R16	1.4964	1.5493	3.3968	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
                              
      2	OUTD3R1	0.025	0.1042	-3.3968	0.0014055	0.0937	2	0	1.5708	0.0000075	0.000015	1
      2	OUTD3R2	0.0999	0.2031	-3.3968	0.0014055	0.0937	2	0	1.5708	0.0000095	0.00003	1
      2	OUTD3R3	0.1988	0.3031	-3.3968	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD3R4	0.3014	0.4026	-3.3968	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD3R5	0.3987	0.5044	-3.3968	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD3R6	0.5035	0.6036	-3.3968	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD3R7	0.599	0.7032	-3.3968	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD3R8	0.7016	0.8056	-3.3968	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD3R9	0.7989	0.9027	-3.3968	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD3R10	0.9017	1.0054	-3.3968	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD3R11	0.9979	1.1015	-3.3968	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD3R12	1.1009	1.2044	-3.3968	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD3R13	1.1959	1.2993	-3.3968	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD3R14	1.2991	1.4024	-3.3968	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD3R15	1.393	1.4963	-3.3968	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD3R16	1.4964	1.5493	-3.3968	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1

      2	OUTD4R1	0.025	0.1042	3.864	0.0014055	0.0937	2	0	1.5708	7.50E-06	0.000015	1
      2	OUTD4R2	0.0999	0.2031	3.864	0.0014055	0.0937	2	0	1.5708	9.50E-06	0.00003	1
      2	OUTD4R3	0.1988	0.3031	3.864	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD4R4	0.3014	0.4026	3.864	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD4R5	0.3987	0.5044	3.864	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD4R6	0.5035	0.6036	3.864	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD4R7	0.599	0.7032	3.864	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD4R8	0.7016	0.8056	3.864	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD4R9	0.7989	0.9027	3.864	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD4R10	0.9017	1.0054	3.864	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD4R11	0.9979	1.1015	3.864	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD4R12	1.1009	1.2044	3.864	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD4R13	1.1959	1.2993	3.864	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD4R14	1.2991	1.4024	3.864	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD4R15	1.393	1.4963	3.864	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD4R16	1.4964	1.5493	3.864	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
                              
      2	OUTD4R1	0.025	0.1042	-3.864	0.0014055	0.0937	2	0	1.5708	0.0000075	0.000015	1
      2	OUTD4R2	0.0999	0.2031	-3.864	0.0014055	0.0937	2	0	1.5708	0.0000095	0.00003	1
      2	OUTD4R3	0.1988	0.3031	-3.864	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD4R4	0.3014	0.4026	-3.864	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD4R5	0.3987	0.5044	-3.864	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD4R6	0.5035	0.6036	-3.864	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD4R7	0.599	0.7032	-3.864	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD4R8	0.7016	0.8056	-3.864	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD4R9	0.7989	0.9027	-3.864	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD4R10	0.9017	1.0054	-3.864	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD4R11	0.9979	1.1015	-3.864	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD4R12	1.1009	1.2044	-3.864	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD4R13	1.1959	1.2993	-3.864	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD4R14	1.2991	1.4024	-3.864	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD4R15	1.393	1.4963	-3.864	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD4R16	1.4964	1.5493	-3.864	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1

      2	OUTD5R1	0.025	0.1042	4.3954	0.0014055	0.0937	2	0	1.5708	7.50E-06	0.000015	1
      2	OUTD5R2	0.0999	0.2031	4.3954	0.0014055	0.0937	2	0	1.5708	9.50E-06	0.00003	1
      2	OUTD5R3	0.1988	0.3031	4.3954	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD5R4	0.3014	0.4026	4.3954	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD5R5	0.3987	0.5044	4.3954	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD5R6	0.5035	0.6036	4.3954	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD5R7	0.599	0.7032	4.3954	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD5R8	0.7016	0.8056	4.3954	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD5R9	0.7989	0.9027	4.3954	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD5R10	0.9017	1.0054	4.3954	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD5R11	0.9979	1.1015	4.3954	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD5R12	1.1009	1.2044	4.3954	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD5R13	1.1959	1.2993	4.3954	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD5R14	1.2991	1.4024	4.3954	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD5R15	1.393	1.4963	4.3954	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD5R16	1.4964	1.5493	4.3954	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
                              
      2	OUTD5R1	0.025	0.1042	-4.3954	0.0014055	0.0937	2	0	1.5708	0.0000075	0.000015	1
      2	OUTD5R2	0.0999	0.2031	-4.3954	0.0014055	0.0937	2	0	1.5708	0.0000095	0.00003	1
      2	OUTD5R3	0.1988	0.3031	-4.3954	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD5R4	0.3014	0.4026	-4.3954	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD5R5	0.3987	0.5044	-4.3954	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD5R6	0.5035	0.6036	-4.3954	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD5R7	0.599	0.7032	-4.3954	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD5R8	0.7016	0.8056	-4.3954	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD5R9	0.7989	0.9027	-4.3954	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD5R10	0.9017	1.0054	-4.3954	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD5R11	0.9979	1.1015	-4.3954	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD5R12	1.1009	1.2044	-4.3954	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD5R13	1.1959	1.2993	-4.3954	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD5R14	1.2991	1.4024	-4.3954	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD5R15	1.393	1.4963	-4.3954	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD5R16	1.4964	1.5493	-4.3954	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1

      2	OUTD6R1	0.025	0.1042	5	0.0014055	0.0937	2	0	1.5708	7.50E-06	0.000015	1
      2	OUTD6R2	0.0999	0.2031	5	0.0014055	0.0937	2	0	1.5708	9.50E-06	0.00003	1
      2	OUTD6R3	0.1988	0.3031	5	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD6R4	0.3014	0.4026	5	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD6R5	0.3987	0.5044	5	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD6R6	0.5035	0.6036	5	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD6R7	0.599	0.7032	5	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD6R8	0.7016	0.8056	5	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD6R9	0.7989	0.9027	5	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	OUTD6R10	0.9017	1.0054	5	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD6R11	0.9979	1.1015	5	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD6R12	1.1009	1.2044	5	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD6R13	1.1959	1.2993	5	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD6R14	1.2991	1.4024	5	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD6R15	1.393	1.4963	5	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	OUTD6R16	1.4964	1.5493	5	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
                              
      2	OUTD6R1	0.025	0.1042	-5	0.0014055	0.0937	2	0	1.5708	0.0000075	0.000015	1
      2	OUTD6R2	0.0999	0.2031	-5	0.0014055	0.0937	2	0	1.5708	0.0000095	0.00003	1
      2	OUTD6R3	0.1988	0.3031	-5	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD6R4	0.3014	0.4026	-5	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD6R5	0.3987	0.5044	-5	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD6R6	0.5035	0.6036	-5	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD6R7	0.599	0.7032	-5	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD6R8	0.7016	0.8056	-5	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD6R9	0.7989	0.9027	-5	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	OUTD6R10	0.9017	1.0054	-5	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD6R11	0.9979	1.1015	-5	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD6R12	1.1009	1.2044	-5	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD6R13	1.1959	1.2993	-5	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD6R14	1.2991	1.4024	-5	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD6R15	1.393	1.4963	-5	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	OUTD6R16	1.4964	1.5493	-5	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1

      # Forward 
      # Barrel/endcap	Name	R min (m)	R max (m)	Z (m)	w (m)	X0	n_meas	th_up	th_down	reso_up	reso_down	flag
      2	FWDlC1R1	0.025	0.1206	6.25	0.0014055	0.0937	2	0	1.5708	7.50E-06	0.000015	1
      2	FWDlC1R2	0.1158	0.2193	6.25	0.0014055	0.0937	2	0	1.5708	9.50E-06	0.00003	1
      2	FWDlC1R3	0.2155	0.3175	6.25	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDlC1R4	0.3156	0.417	6.25	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDlC1R5	0.4144	0.5154	6.25	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDlC1R6	0.5144	0.6152	6.25	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDlC1R7	0.6125	0.7167	6.25	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDlC1R8	0.7148	0.8188	6.25	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDlC1R9	0.8147	0.9186	6.25	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
                              
      2	FWDlC1R1	0.025	0.1206	-6.25	0.0014055	0.0937	2	0	1.5708	0.0000075	0.000015	1
      2	FWDlC1R2	0.1158	0.2193	-6.25	0.0014055	0.0937	2	0	1.5708	0.0000095	0.00003	1
      2	FWDlC1R3	0.2155	0.3175	-6.25	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDlC1R4	0.3156	0.417	-6.25	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDlC1R5	0.4144	0.5154	-6.25	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDlC1R6	0.5144	0.6152	-6.25	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDlC1R7	0.6125	0.7167	-6.25	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDlC1R8	0.7148	0.8188	-6.25	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDlC1R9	0.8147	0.9186	-6.25	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1

      2	FWDlC2R1	0.025	0.1206	7.3951	0.0014055	0.0937	2	0	1.5708	7.50E-06	0.000015	1
      2	FWDlC2R2	0.1158	0.2193	7.3951	0.0014055	0.0937	2	0	1.5708	9.50E-06	0.00003	1
      2	FWDlC2R3	0.2155	0.3175	7.3951	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDlC2R4	0.3156	0.417	7.3951	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDlC2R5	0.4144	0.5154	7.3951	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDlC2R6	0.5144	0.6152	7.3951	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDlC2R7	0.6125	0.7167	7.3951	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDlC2R8	0.7148	0.8188	7.3951	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDlC2R9	0.8147	0.9186	7.3951	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDlC2R10	0.9166	1.0206	7.3951	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDlC2R11	1.0166	1.1206	7.3951	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
                              
      2	FWDlC2R1	0.025	0.1206	-7.3951	0.0014055	0.0937	2	0	1.5708	0.0000075	0.000015	1
      2	FWDlC2R2	0.1158	0.2193	-7.3951	0.0014055	0.0937	2	0	1.5708	0.0000095	0.00003	1
      2	FWDlC2R3	0.2155	0.3175	-7.3951	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDlC2R4	0.3156	0.417	-7.3951	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDlC2R5	0.4144	0.5154	-7.3951	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDlC2R6	0.5144	0.6152	-7.3951	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDlC2R7	0.6125	0.7167	-7.3951	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDlC2R8	0.7148	0.8188	-7.3951	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDlC2R9	0.8147	0.9186	-7.3951	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDlC2R10	0.9166	1.0206	-7.3951	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDlC2R11	1.0166	1.1206	-7.3951	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1

      2	FWDlC3R1	0.025	0.1206	8.75	0.0014055	0.0937	2	0	1.5708	7.50E-06	0.000015	1
      2	FWDlC3R2	0.1158	0.2193	8.75	0.0014055	0.0937	2	0	1.5708	9.50E-06	0.00003	1
      2	FWDlC3R3	0.2155	0.3175	8.75	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDlC3R4	0.3156	0.417	8.75	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDlC3R5	0.4144	0.5154	8.75	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDlC3R6	0.5144	0.6152	8.75	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDlC3R7	0.6125	0.7167	8.75	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDlC3R8	0.7148	0.8188	8.75	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDlC3R9	0.8147	0.9186	8.75	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDlC3R10	0.9166	1.0206	8.75	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDlC3R11	1.0166	1.1206	8.75	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDlC3R12	1.1166	1.2166	8.75	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDlC3R13	1.2126	1.3215	8.75	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
                              
      2	FWDlC3R1	0.025	0.1206	-8.75	0.0014055	0.0937	2	0	1.5708	0.0000075	0.000015	1
      2	FWDlC3R2	0.1158	0.2193	-8.75	0.0014055	0.0937	2	0	1.5708	0.0000095	0.00003	1
      2	FWDlC3R3	0.2155	0.3175	-8.75	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDlC3R4	0.3156	0.417	-8.75	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDlC3R5	0.4144	0.5154	-8.75	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDlC3R6	0.5144	0.6152	-8.75	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDlC3R7	0.6125	0.7167	-8.75	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDlC3R8	0.7148	0.8188	-8.75	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDlC3R9	0.8147	0.9186	-8.75	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDlC3R10	0.9166	1.0206	-8.75	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDlC3R11	1.0166	1.1206	-8.75	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDlC3R12	1.1166	1.2166	-8.75	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDlC3R13	1.2126	1.3215	-8.75	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1

      2	FWDC1R1	0.0495	0.1446	10	0.0014055	0.0937	2	0	1.5708	7.50E-06	0.000015	1
      2	FWDC1R2	0.1391	0.2412	10	0.0014055	0.0937	2	0	1.5708	9.50E-06	0.00003	1
      2	FWDC1R3	0.2382	0.3396	10	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC1R4	0.3378	0.4388	10	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC1R5	0.4367	0.5375	10	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC1R6	0.5363	0.6408	10	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC1R7	0.6373	0.7414	10	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC1R8	0.7394	0.8434	10	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC1R9	0.84	0.9438	10	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC1R10	0.9422	1.0459	10	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDC1R11	1.0426	1.1461	10	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDC1R12	1.1448	1.2483	10	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDC1R13	1.2449	1.3482	10	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDC1R14	1.3472	1.4505	10	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDC1R15	1.4469	1.5502	10	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
                              
      2	FWDC1R1	0.0495	0.1446	-10	0.0014055	0.0937	2	0	1.5708	0.0000075	0.000015	1
      2	FWDC1R2	0.1391	0.2412	-10	0.0014055	0.0937	2	0	1.5708	0.0000095	0.00003	1
      2	FWDC1R3	0.2382	0.3396	-10	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC1R4	0.3378	0.4388	-10	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC1R5	0.4367	0.5375	-10	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC1R6	0.5363	0.6408	-10	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC1R7	0.6373	0.7414	-10	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC1R8	0.7394	0.8434	-10	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC1R9	0.84	0.9438	-10	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC1R10	0.9422	1.0459	-10	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDC1R11	1.0426	1.1461	-10	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDC1R12	1.1448	1.2483	-10	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDC1R13	1.2449	1.3482	-10	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDC1R14	1.3472	1.4505	-10	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDC1R15	1.4469	1.5502	-10	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1

      2	FWDC2R1	0.0495	0.1446	10.9856	0.0014055	0.0937	2	0	1.5708	7.50E-06	0.000015	1
      2	FWDC2R2	0.1391	0.2412	10.9856	0.0014055	0.0937	2	0	1.5708	9.50E-06	0.00003	1
      2	FWDC2R3	0.2382	0.3396	10.9856	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC2R4	0.3378	0.4388	10.9856	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC2R5	0.4367	0.5375	10.9856	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC2R6	0.5363	0.6408	10.9856	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC2R7	0.6373	0.7414	10.9856	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC2R8	0.7394	0.8434	10.9856	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC2R9	0.84	0.9438	10.9856	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC2R10	0.9422	1.0459	10.9856	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDC2R11	1.0426	1.1461	10.9856	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDC2R12	1.1448	1.2483	10.9856	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDC2R13	1.2449	1.3482	10.9856	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDC2R14	1.3472	1.4505	10.9856	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDC2R15	1.4469	1.5502	10.9856	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
                              
      2	FWDC2R1	0.0495	0.1446	-10.9856	0.0014055	0.0937	2	0	1.5708	0.0000075	0.000015	1
      2	FWDC2R2	0.1391	0.2412	-10.9856	0.0014055	0.0937	2	0	1.5708	0.0000095	0.00003	1
      2	FWDC2R3	0.2382	0.3396	-10.9856	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC2R4	0.3378	0.4388	-10.9856	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC2R5	0.4367	0.5375	-10.9856	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC2R6	0.5363	0.6408	-10.9856	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC2R7	0.6373	0.7414	-10.9856	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC2R8	0.7394	0.8434	-10.9856	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC2R9	0.84	0.9438	-10.9856	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC2R10	0.9422	1.0459	-10.9856	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDC2R11	1.0426	1.1461	-10.9856	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDC2R12	1.1448	1.2483	-10.9856	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDC2R13	1.2449	1.3482	-10.9856	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDC2R14	1.3472	1.4505	-10.9856	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDC2R15	1.4469	1.5502	-10.9856	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1

      2	FWDC3R1	0.0495	0.1446	12.0684	0.0014055	0.0937	2	0	1.5708	7.50E-06	0.000015	1
      2	FWDC3R2	0.1391	0.2412	12.0684	0.0014055	0.0937	2	0	1.5708	9.50E-06	0.00003	1
      2	FWDC3R3	0.2382	0.3396	12.0684	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC3R4	0.3378	0.4388	12.0684	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC3R5	0.4367	0.5375	12.0684	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC3R6	0.5363	0.6408	12.0684	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC3R7	0.6373	0.7414	12.0684	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC3R8	0.7394	0.8434	12.0684	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC3R9	0.84	0.9438	12.0684	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC3R10	0.9422	1.0459	12.0684	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDC3R11	1.0426	1.1461	12.0684	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDC3R12	1.1448	1.2483	12.0684	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDC3R13	1.2449	1.3482	12.0684	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDC3R14	1.3472	1.4505	12.0684	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDC3R15	1.4469	1.5502	12.0684	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
                              
      2	FWDC3R1	0.0495	0.1446	-12.0684	0.0014055	0.0937	2	0	1.5708	0.0000075	0.000015	1
      2	FWDC3R2	0.1391	0.2412	-12.0684	0.0014055	0.0937	2	0	1.5708	0.0000095	0.00003	1
      2	FWDC3R3	0.2382	0.3396	-12.0684	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC3R4	0.3378	0.4388	-12.0684	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC3R5	0.4367	0.5375	-12.0684	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC3R6	0.5363	0.6408	-12.0684	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC3R7	0.6373	0.7414	-12.0684	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC3R8	0.7394	0.8434	-12.0684	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC3R9	0.84	0.9438	-12.0684	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC3R10	0.9422	1.0459	-12.0684	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDC3R11	1.0426	1.1461	-12.0684	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDC3R12	1.1448	1.2483	-12.0684	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDC3R13	1.2449	1.3482	-12.0684	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDC3R14	1.3472	1.4505	-12.0684	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDC3R15	1.4469	1.5502	-12.0684	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1

      2	FWDC4R1	0.0495	0.1446	13.2578	0.0014055	0.0937	2	0	1.5708	7.50E-06	0.000015	1
      2	FWDC4R2	0.1391	0.2412	13.2578	0.0014055	0.0937	2	0	1.5708	9.50E-06	0.00003	1
      2	FWDC4R3	0.2382	0.3396	13.2578	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC4R4	0.3378	0.4388	13.2578	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC4R5	0.4367	0.5375	13.2578	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC4R6	0.5363	0.6408	13.2578	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC4R7	0.6373	0.7414	13.2578	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC4R8	0.7394	0.8434	13.2578	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC4R9	0.84	0.9438	13.2578	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC4R10	0.9422	1.0459	13.2578	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDC4R11	1.0426	1.1461	13.2578	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDC4R12	1.1448	1.2483	13.2578	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDC4R13	1.2449	1.3482	13.2578	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDC4R14	1.3472	1.4505	13.2578	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDC4R15	1.4469	1.5502	13.2578	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
                              
      2	FWDC4R1	0.0495	0.1446	-13.2578	0.0014055	0.0937	2	0	1.5708	0.0000075	0.000015	1
      2	FWDC4R2	0.1391	0.2412	-13.2578	0.0014055	0.0937	2	0	1.5708	0.0000095	0.00003	1
      2	FWDC4R3	0.2382	0.3396	-13.2578	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC4R4	0.3378	0.4388	-13.2578	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC4R5	0.4367	0.5375	-13.2578	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC4R6	0.5363	0.6408	-13.2578	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC4R7	0.6373	0.7414	-13.2578	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC4R8	0.7394	0.8434	-13.2578	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC4R9	0.84	0.9438	-13.2578	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC4R10	0.9422	1.0459	-13.2578	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDC4R11	1.0426	1.1461	-13.2578	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDC4R12	1.1448	1.2483	-13.2578	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDC4R13	1.2449	1.3482	-13.2578	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDC4R14	1.3472	1.4505	-13.2578	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDC4R15	1.4469	1.5502	-13.2578	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1

      2	FWDC5R1	0.0495	0.1446	14.5645	0.0014055	0.0937	2	0	1.5708	7.50E-06	0.000015	1
      2	FWDC5R2	0.1391	0.2412	14.5645	0.0014055	0.0937	2	0	1.5708	9.50E-06	0.00003	1
      2	FWDC5R3	0.2382	0.3396	14.5645	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC5R4	0.3378	0.4388	14.5645	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC5R5	0.4367	0.5375	14.5645	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC5R6	0.5363	0.6408	14.5645	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC5R7	0.6373	0.7414	14.5645	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC5R8	0.7394	0.8434	14.5645	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC5R9	0.84	0.9438	14.5645	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC5R10	0.9422	1.0459	14.5645	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDC5R11	1.0426	1.1461	14.5645	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDC5R12	1.1448	1.2483	14.5645	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDC5R13	1.2449	1.3482	14.5645	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDC5R14	1.3472	1.4505	14.5645	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDC5R15	1.4469	1.5502	14.5645	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
                              
      2	FWDC5R1	0.0495	0.1446	-14.5645	0.0014055	0.0937	2	0	1.5708	0.0000075	0.000015	1
      2	FWDC5R2	0.1391	0.2412	-14.5645	0.0014055	0.0937	2	0	1.5708	0.0000095	0.00003	1
      2	FWDC5R3	0.2382	0.3396	-14.5645	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC5R4	0.3378	0.4388	-14.5645	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC5R5	0.4367	0.5375	-14.5645	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC5R6	0.5363	0.6408	-14.5645	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC5R7	0.6373	0.7414	-14.5645	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC5R8	0.7394	0.8434	-14.5645	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC5R9	0.84	0.9438	-14.5645	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC5R10	0.9422	1.0459	-14.5645	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDC5R11	1.0426	1.1461	-14.5645	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDC5R12	1.1448	1.2483	-14.5645	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDC5R13	1.2449	1.3482	-14.5645	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDC5R14	1.3472	1.4505	-14.5645	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDC5R15	1.4469	1.5502	-14.5645	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1

      2	FWDC6R1	0.0495	0.1446	16	0.0014055	0.0937	2	0	1.5708	7.50E-06	0.000015	1
      2	FWDC6R2	0.1391	0.2412	16	0.0014055	0.0937	2	0	1.5708	9.50E-06	0.00003	1
      2	FWDC6R3	0.2382	0.3396	16	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC6R4	0.3378	0.4388	16	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC6R5	0.4367	0.5375	16	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC6R6	0.5363	0.6408	16	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC6R7	0.6373	0.7414	16	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC6R8	0.7394	0.8434	16	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC6R9	0.84	0.9438	16	0.001874	0.0937	2	0	1.5708	9.50E-06	0.000115	1
      2	FWDC6R10	0.9422	1.0459	16	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDC6R11	1.0426	1.1461	16	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDC6R12	1.1448	1.2483	16	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDC6R13	1.2449	1.3482	16	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDC6R14	1.3472	1.4505	16	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
      2	FWDC6R15	1.4469	1.5502	16	0.0023425	0.0937	2	0	1.5708	9.50E-06	0.002887	1
                              
      2	FWDC6R1	0.0495	0.1446	-16	0.0014055	0.0937	2	0	1.5708	0.0000075	0.000015	1
      2	FWDC6R2	0.1391	0.2412	-16	0.0014055	0.0937	2	0	1.5708	0.0000095	0.00003	1
      2	FWDC6R3	0.2382	0.3396	-16	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC6R4	0.3378	0.4388	-16	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC6R5	0.4367	0.5375	-16	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC6R6	0.5363	0.6408	-16	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC6R7	0.6373	0.7414	-16	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC6R8	0.7394	0.8434	-16	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC6R9	0.84	0.9438	-16	0.001874	0.0937	2	0	1.5708	0.0000095	0.000115	1
      2	FWDC6R10	0.9422	1.0459	-16	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDC6R11	1.0426	1.1461	-16	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDC6R12	1.1448	1.2483	-16	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDC6R13	1.2449	1.3482	-16	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDC6R14	1.3472	1.4505	-16	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
      2	FWDC6R15	1.4469	1.5502	-16	0.0023425	0.0937	2	0	1.5708	0.0000095	0.002887	1
    }

}
##############
# Track merger
##############

module Merger TrackMerger {
# add InputArray InputArray
  add InputArray TrackSmearing/tracks
  set OutputArray tracks
}

######################
# Looper Selection
######################

module Efficiency ForwardLooperTracks  {
  set InputArray TrackMerger/tracks
  set OutputArray tracks
  set UseMomentumVector False

  ## select looping tracks that end up in position |eta| > 6.000 (lost by calo)
  set EfficiencyFormula {
    (abs(eta) > 6.0 )                                 * (1.000) +
    (abs(eta) <= 6.0 )                                * (0.000)
  }

}

#############
# Calorimeter
#############
module DualReadoutCalorimeter Calorimeter {
  set ParticleInputArray ParticlePropagator/stableParticles
  set TrackInputArray TrackMerger/tracks

  set TowerOutputArray towers
  set PhotonOutputArray photons

  set EFlowTrackOutputArray eflowTracks
  set EFlowPhotonOutputArray eflowPhotons
  set EFlowNeutralHadronOutputArray eflowNeutralHadrons

  set ECalMinSignificance 2.0
  set HCalMinSignificance 2.5

  set SmearLogNormal false

  set SmearTowerCenter true
  #set SmearTowerCenter false
    set pi [expr {acos(-1)}]

    # Lists of the edges of each tower in eta and phi;
    # each list starts with the lower edge of the first tower;
    # the list ends with the higher edged of the last tower.
    # Barrel:  deta=0.02 towers up to |eta| <= 0.88 ( up to 45°)
    # Endcaps: deta=0.02 towers up to |eta| <= 3.0 (8.6° = 100 mrad)
    # Cell size: about 6 cm x 6 cm

    set EtaPhiRes 0.02
    set EtaMax 6.0

    set pi [expr {acos(-1)}]

    set nbins_phi [expr {$pi/$EtaPhiRes} ]
    set nbins_phi [expr {int($nbins_phi)} ]

    set PhiBins {}
    for {set i -$nbins_phi} {$i <= $nbins_phi} {incr i} {
      add PhiBins [expr {$i * $pi/$nbins_phi}]
    }

    set nbins_eta [expr {$EtaMax/$EtaPhiRes} ]
    set nbins_eta [expr {int($nbins_eta)} ]

    for {set i -$nbins_eta} {$i <= $nbins_eta} {incr i} {
      set eta [expr {$i * $EtaPhiRes}]
      add EtaPhiBins $eta $PhiBins
    }

    # default energy fractions {abs(PDG code)} {Fecal Fhcal}
    add EnergyFraction {0} {0.0 1.0}
    # energy fractions for e, gamma and pi0
    add EnergyFraction {11} {1.0 0.0}
    add EnergyFraction {22} {1.0 0.0}
    add EnergyFraction {111} {1.0 0.0}
    # energy fractions for muon, neutrinos and neutralinos
    add EnergyFraction {12} {0.0 0.0}
    add EnergyFraction {13} {0.0 0.0}
    add EnergyFraction {14} {0.0 0.0}
    add EnergyFraction {16} {0.0 0.0}
    add EnergyFraction {1000022} {0.0 0.0}
    add EnergyFraction {1000023} {0.0 0.0}
    add EnergyFraction {1000025} {0.0 0.0}
    add EnergyFraction {1000035} {0.0 0.0}
    add EnergyFraction {1000045} {0.0 0.0}
    # energy fractions for K0short and Lambda
    add EnergyFraction {310} {0.3 0.7}
    add EnergyFraction {130} {0.3 0.7}
    add EnergyFraction {3122} {0.3 0.7}


    ## ECAL crystals for the EM part from 2008.00338
    # set ECalResolutionFormula {resolution formula as a function of eta and energy}
    set ECalResolutionFormula {
    (abs(eta) <= 4 )                     * sqrt(energy^2*0.0015^2 + energy*0.082^2 + 0.31^2)+
    (abs(eta) > 4 && abs(eta) <= 6.0)    * sqrt(energy^2*0.035^2  + energy*0.30^2 + 0.31^2)
    }


    # Dual Readout
    # set HCalResolutionFormula {resolution formula as a function of eta and energy}
    set HCalResolutionFormula {
    (abs(eta) <= 4 )                     * sqrt(energy^2*0.01^2 + energy*0.48^2 + 0.05^2)+
    (abs(eta) > 4 && abs(eta) <= 6.0)    * sqrt(energy^2*0.01^2 + energy*1.00^2 + 0.05^2)
    }
}



#################
# Electron filter
#################

module PdgCodeFilter ElectronFilter {
  set InputArray EFlowTrackMerger/eflowTracks
  set OutputArray electrons
  set Invert true
  add PdgCode {11}
  add PdgCode {-11}
}


#################
# Muon filter
#################

module PdgCodeFilter MuonFilter {
  set InputArray EFlowTrackMerger/eflowTracks
  set OutputArray muons
  set Invert true
  add PdgCode {13}
  add PdgCode {-13}
}



###################################################
# Tower Merger (in case not using e-flow algorithm)
###################################################

module Merger TowerMerger {
# add InputArray InputArray
  add InputArray Calorimeter/towers
  add InputArray MuonFilter/muons
  set OutputArray towers
}



############################
# Energy flow track merger
############################

module Merger EFlowTrackMerger {
# add InputArray InputArray
  add InputArray Calorimeter/eflowTracks
  add InputArray ForwardLooperTracks/tracks
  set OutputArray eflowTracks
}


####################
# Energy flow merger
####################

module Merger EFlowMerger {
# add InputArray InputArray
  add InputArray EFlowTrackMerger/eflowTracks
  add InputArray Calorimeter/eflowPhotons
  add InputArray Calorimeter/eflowNeutralHadrons
  set OutputArray eflow
}


######################
# EFlowFilter
######################

module PdgCodeFilter EFlowFilter {
  set InputArray EFlowMerger/eflow
  set OutputArray eflow

  add PdgCode {11}
  add PdgCode {-11}
  add PdgCode {13}
  add PdgCode {-13}
}


###################
# Missing ET merger
###################

module Merger MissingET {
# add InputArray InputArray
  add InputArray EFlowMerger/eflow
  set MomentumOutputArray momentum
}



##################
# Scalar HT merger
##################

module Merger ScalarHT {
# add InputArray InputArray
  add InputArray EFlowMerger/eflow
  set EnergyOutputArray energy
}

#################
# Neutrino Filter
#################

module PdgCodeFilter NeutrinoFilter {

  set InputArray Delphes/stableParticles
  set OutputArray filteredParticles

  set PTMin 0.0

  add PdgCode {12}
  add PdgCode {14}
  add PdgCode {16}
  add PdgCode {-12}
  add PdgCode {-14}
  add PdgCode {-16}

}


#########################
# Gen Missing ET merger
########################

module Merger GenMissingET {

# add InputArray InputArray
  add InputArray NeutrinoFilter/filteredParticles
  set MomentumOutputArray momentum
}



#####################
# MC truth jet finder
#####################

# TBC: is jet radius fine?

module FastJetFinder GenJetFinder02 {
  set InputArray NeutrinoFilter/filteredParticles

  set OutputArray jets

  # algorithm: 1 CDFJetClu, 2 MidPoint, 3 SIScone, 4 kt, 5 Cambridge/Aachen, 6 antikt
  set JetAlgorithm 6
  set ParameterR 0.2

  set ComputeNsubjettiness 1
  set Beta 1.0
  set AxisMode 4

  set ComputeSoftDrop 1
  set BetaSoftDrop 0.0
  set SymmetryCutSoftDrop 0.1
  set R0SoftDrop 0.2

  set JetPTMin 25.0
}


#####################
# MC truth jet finder
#####################

# TBC: is jet radius fine?

module FastJetFinder GenJetFinder04 {
  set InputArray NeutrinoFilter/filteredParticles

  set OutputArray jets

  # algorithm: 1 CDFJetClu, 2 MidPoint, 3 SIScone, 4 kt, 5 Cambridge/Aachen, 6 antikt
  set JetAlgorithm 6
  set ParameterR 0.4

  set ComputeNsubjettiness 1
  set Beta 1.0
  set AxisMode 4

  set ComputeSoftDrop 1
  set BetaSoftDrop 0.0
  set SymmetryCutSoftDrop 0.1
  set R0SoftDrop 0.4

  set JetPTMin 25.0
}

#####################
# MC truth jet finder
#####################

# TBC: is jet radius fine?

module FastJetFinder GenJetFinder08 {
  set InputArray NeutrinoFilter/filteredParticles

  set OutputArray jets

  # algorithm: 1 CDFJetClu, 2 MidPoint, 3 SIScone, 4 kt, 5 Cambridge/Aachen, 6 antikt
  set JetAlgorithm 6
  set ParameterR 0.8

  set ComputeNsubjettiness 1
  set Beta 1.0
  set AxisMode 4

  set ComputeSoftDrop 1
  set BetaSoftDrop 0.0
  set SymmetryCutSoftDrop 0.1
  set R0SoftDrop 0.8

  set JetPTMin 25.0
}

#####################
# MC truth jet finder
#####################

# TBC: is jet radius fine?

module FastJetFinder GenJetFinder15 {
  set InputArray NeutrinoFilter/filteredParticles

  set OutputArray jets

  # algorithm: 1 CDFJetClu, 2 MidPoint, 3 SIScone, 4 kt, 5 Cambridge/Aachen, 6 antikt
  set JetAlgorithm 6
  set ParameterR 1.5

  set ComputeNsubjettiness 1
  set Beta 1.0
  set AxisMode 4

  set ComputeSoftDrop 1
  set BetaSoftDrop 0.0
  set SymmetryCutSoftDrop 0.1
  set R0SoftDrop 1.5

  set JetPTMin 25.0
}


##################
# Fast Jet finder
##################

module FastJetFinder FastJetFinder02 {
  set InputArray EFlowMerger/eflow

  set OutputArray jets

  # algorithm: 1 CDFJetClu, 2 MidPoint, 3 SIScone, 4 kt, 5 Cambridge/Aachen, 6 antikt
  set JetAlgorithm 6
  set ParameterR 0.2

  set ComputeNsubjettiness 1
  set Beta 1.0
  set AxisMode 4

  set ComputeSoftDrop 1
  set BetaSoftDrop 0.0
  set SymmetryCutSoftDrop 0.1
  set R0SoftDrop 0.2

  set JetPTMin 25.0
}

##################
# Fast Jet finder
##################

module FastJetFinder FastJetFinder04 {
  set InputArray EFlowMerger/eflow

  set OutputArray jets

  # algorithm: 1 CDFJetClu, 2 MidPoint, 3 SIScone, 4 kt, 5 Cambridge/Aachen, 6 antikt
  set JetAlgorithm 6
  set ParameterR 0.4

  set ComputeNsubjettiness 1
  set Beta 1.0
  set AxisMode 4

  set ComputeSoftDrop 1
  set BetaSoftDrop 0.0
  set SymmetryCutSoftDrop 0.1
  set R0SoftDrop 0.4

  set JetPTMin 25.0
}


##################
# Fast Jet finder
##################

module FastJetFinder FastJetFinder08 {
  set InputArray EFlowMerger/eflow

  set OutputArray jets

  # algorithm: 1 CDFJetClu, 2 MidPoint, 3 SIScone, 4 kt, 5 Cambridge/Aachen, 6 antikt
  set JetAlgorithm 6
  set ParameterR 0.8

  set ComputeNsubjettiness 1
  set Beta 1.0
  set AxisMode 4

  set ComputeSoftDrop 1
  set BetaSoftDrop 0.0
  set SymmetryCutSoftDrop 0.1
  set R0SoftDrop 0.8

  set JetPTMin 25.0
}

##################
# Fast Jet finder
##################

module FastJetFinder FastJetFinder15 {
  set InputArray EFlowMerger/eflow

  set OutputArray jets

  # algorithm: 1 CDFJetClu, 2 MidPoint, 3 SIScone, 4 kt, 5 Cambridge/Aachen, 6 antikt
  set JetAlgorithm 6
  set ParameterR 1.5

  set ComputeNsubjettiness 1
  set Beta 1.0
  set AxisMode 4

  set ComputeSoftDrop 1
  set BetaSoftDrop 0.0
  set SymmetryCutSoftDrop 0.1
  set R0SoftDrop 1.5

  set JetPTMin 25.0
}


##################
# Fast Jet finder
##################

module FastJetFinder CaloJetFinder02 {
  set InputArray TowerMerger/towers

  set OutputArray jets

  # algorithm: 1 CDFJetClu, 2 MidPoint, 3 SIScone, 4 kt, 5 Cambridge/Aachen, 6 antikt
  set JetAlgorithm 6
  set ParameterR 0.2

  set ComputeNsubjettiness 1
  set Beta 1.0
  set AxisMode 4

  set ComputeSoftDrop 1
  set BetaSoftDrop 0.0
  set SymmetryCutSoftDrop 0.1
  set R0SoftDrop 0.2

  set JetPTMin 25.0
}

##################
# Fast Jet finder
##################

module FastJetFinder CaloJetFinder04 {
  set InputArray TowerMerger/towers

  set OutputArray jets

  # algorithm: 1 CDFJetClu, 2 MidPoint, 3 SIScone, 4 kt, 5 Cambridge/Aachen, 6 antikt
  set JetAlgorithm 6
  set ParameterR 0.4

  set ComputeNsubjettiness 1
  set Beta 1.0
  set AxisMode 4

  set ComputeSoftDrop 1
  set BetaSoftDrop 0.0
  set SymmetryCutSoftDrop 0.1
  set R0SoftDrop 0.4

  set JetPTMin 25.0
}


##################
# Fast Jet finder
##################

module FastJetFinder CaloJetFinder08 {
  set InputArray TowerMerger/towers

  set OutputArray jets

  # algorithm: 1 CDFJetClu, 2 MidPoint, 3 SIScone, 4 kt, 5 Cambridge/Aachen, 6 antikt
  set JetAlgorithm 6
  set ParameterR 0.8

  set ComputeNsubjettiness 1
  set Beta 1.0
  set AxisMode 4

  set ComputeSoftDrop 1
  set BetaSoftDrop 0.0
  set SymmetryCutSoftDrop 0.1
  set R0SoftDrop 0.8

  set JetPTMin 25.0
}

##################
# Fast Jet finder
##################

module FastJetFinder CaloJetFinder15 {
  set InputArray TowerMerger/towers

  set OutputArray jets

  # algorithm: 1 CDFJetClu, 2 MidPoint, 3 SIScone, 4 kt, 5 Cambridge/Aachen, 6 antikt
  set JetAlgorithm 6
  set ParameterR 1.5

  set ComputeNsubjettiness 1
  set Beta 1.0
  set AxisMode 4

  set ComputeSoftDrop 1
  set BetaSoftDrop 0.0
  set SymmetryCutSoftDrop 0.1
  set R0SoftDrop 1.5

  set JetPTMin 25.0
}


##################
# Fast Jet finder
##################

module FastJetFinder TrackJetFinder02 {
  set InputArray TrackMerger/tracks

  set OutputArray jets

  # algorithm: 1 CDFJetClu, 2 MidPoint, 3 SIScone, 4 kt, 5 Cambridge/Aachen, 6 antikt
  set JetAlgorithm 6
  set ParameterR 0.2

  set ComputeNsubjettiness 1
  set Beta 1.0
  set AxisMode 4

  set ComputeSoftDrop 1
  set BetaSoftDrop 0.0
  set SymmetryCutSoftDrop 0.1
  set R0SoftDrop 0.2

  set JetPTMin 25.0
}

##################
# Fast Jet finder
##################

module FastJetFinder TrackJetFinder04 {
  set InputArray TrackMerger/tracks

  set OutputArray jets

  # algorithm: 1 CDFJetClu, 2 MidPoint, 3 SIScone, 4 kt, 5 Cambridge/Aachen, 6 antikt
  set JetAlgorithm 6
  set ParameterR 0.4

  set ComputeNsubjettiness 1
  set Beta 1.0
  set AxisMode 4

  set ComputeSoftDrop 1
  set BetaSoftDrop 0.0
  set SymmetryCutSoftDrop 0.1
  set R0SoftDrop 0.4

  set JetPTMin 25.0
}


##################
# Fast Jet finder
##################

module FastJetFinder TrackJetFinder08 {
  set InputArray TrackMerger/tracks

  set OutputArray jets

  # algorithm: 1 CDFJetClu, 2 MidPoint, 3 SIScone, 4 kt, 5 Cambridge/Aachen, 6 antikt
  set JetAlgorithm 6
  set ParameterR 0.8

  set ComputeNsubjettiness 1
  set Beta 1.0
  set AxisMode 4

  set ComputeSoftDrop 1
  set BetaSoftDrop 0.0
  set SymmetryCutSoftDrop 0.1
  set R0SoftDrop 0.8

  set JetPTMin 25.0
}

##################
# Fast Jet finder
##################

module FastJetFinder TrackJetFinder15 {
  set InputArray TrackMerger/tracks

  set OutputArray jets

  # algorithm: 1 CDFJetClu, 2 MidPoint, 3 SIScone, 4 kt, 5 Cambridge/Aachen, 6 antikt
  set JetAlgorithm 6
  set ParameterR 1.5

  set ComputeNsubjettiness 1
  set Beta 1.0
  set AxisMode 4

  set ComputeSoftDrop 1
  set BetaSoftDrop 0.0
  set SymmetryCutSoftDrop 0.1
  set R0SoftDrop 1.5

  set JetPTMin 25.0
}


##################
# Jet Energy Scale
##################

module EnergyScale JetEnergyScale {
  set InputArray FastJetFinder04/jets
  set OutputArray jets

 # scale formula for jets
  set ScaleFormula {1.00}
}


########################
# Jet Flavor Association
########################

module JetFlavorAssociation JetFlavorAssociation {

  set PartonInputArray Delphes/partons
  set ParticleInputArray Delphes/allParticles
  set ParticleLHEFInputArray Delphes/allParticlesLHEF
  set JetInputArray JetEnergyScale/jets

  set DeltaR 0.5
  set PartonPTMin 5.0
  set PartonEtaMax 6.0

}

###################
# Photon efficiency
###################

module Efficiency PhotonEfficiency {
  set InputArray Calorimeter/eflowPhotons
  set OutputArray photons

  # set EfficiencyFormula {efficiency formula as a function of eta and pt}

  set EfficiencyFormula {
  0.95 * (
  (pt <= 1.0) * (0.00) + \
  (abs(eta) <= 2.5) * (pt > 1.0 && pt < 5.0)  * (0.80) +
  (abs(eta) <= 2.5) * (pt > 5.0 && pt < 10.0) * (0.90) +
  (abs(eta) <= 2.5) * (pt > 10.0)             * (0.95) +

  (abs(eta) > 2.5 && abs(eta) <= 4.0) * (pt > 1.0 && pt < 5.0)  * (0.80) +
  (abs(eta) > 2.5 && abs(eta) <= 4.0) * (pt > 5.0 && pt < 10.0) * (0.85) +
  (abs(eta) > 2.5 && abs(eta) <= 4.0) * (pt > 10.0)		* (0.90) +

  (abs(eta) > 4.0 && abs(eta) <= 6.0) * (pt > 1.0 && pt < 5.0)  * (0.75) + \
  (abs(eta) > 4.0 && abs(eta) <= 6.0) * (pt > 5.0 && pt < 10.0) * (0.80) + \
  (abs(eta) > 4.0 && abs(eta) <= 6.0) * (pt > 10.0)             * (0.85) + \
  (abs(eta) > 6.0) * (0.00)
  )
  }

}

##################
# Photon isolation
##################

# TBC: check values for iso cuts

module Isolation PhotonIsolation {
  set CandidateInputArray PhotonEfficiency/photons
  set IsolationInputArray EFlowFilter/eflow

  set OutputArray photons

  # veto isolation cand. based on proximity to input cand. -> turned out not to be needed
  # set DeltaRMin 0.02
  # set UseMiniCone true

  set DeltaRMax 0.3

  set PTMin 0.5

  set PTRatioMax 0.1
}


###################
# Electron efficiency
###################

module Efficiency ElectronEfficiency {
  set InputArray ElectronFilter/electrons
  set OutputArray electrons

  # set EfficiencyFormula {efficiency formula as a function of eta and pt}

  set EfficiencyFormula {
  (pt <= 1.0) * (0.00) + \
  (abs(eta) <= 2.5) * (pt > 1.0 && pt < 5.0)  * (0.80) +
  (abs(eta) <= 2.5) * (pt > 5.0 && pt < 10.0) * (0.90) +
  (abs(eta) <= 2.5) * (pt > 10.0)             * (0.95) +

  (abs(eta) > 2.5 && abs(eta) <= 4.0) * (pt > 1.0 && pt < 5.0)  * (0.80) +
  (abs(eta) > 2.5 && abs(eta) <= 4.0) * (pt > 5.0 && pt < 10.0) * (0.85) +
  (abs(eta) > 2.5 && abs(eta) <= 4.0) * (pt > 10.0)		* (0.90) +

  (abs(eta) > 4.0 && abs(eta) <= 6.0) * (pt > 1.0 && pt < 5.0)  * (0.75) + \
  (abs(eta) > 4.0 && abs(eta) <= 6.0) * (pt > 5.0 && pt < 10.0) * (0.80) + \
  (abs(eta) > 4.0 && abs(eta) <= 6.0) * (pt > 10.0)             * (0.85) + \
  (abs(eta) > 6.0) * (0.00)
  }

}


####################
# Electron isolation
####################

# TBC: check values for iso cuts

module Isolation ElectronIsolation {
  set CandidateInputArray ElectronEfficiency/electrons
  set IsolationInputArray EFlowFilter/eflow

  set OutputArray electrons

  # veto isolation cand. based on proximity to input cand. -> turned out not to be needed
  # set DeltaRMin 0.02
  # set UseMiniCone true

  set DeltaRMax 0.3

  set PTMin 0.5

  set PTRatioMax 0.1
}




###################
# Muon efficiency
###################

module Efficiency MuonEfficiency {
  set InputArray MuonFilter/muons
  set OutputArray muons

  # set EfficiencyFormula {efficiency formula as a function of eta and pt}

  set EfficiencyFormula {
  (pt <= 1.0) * (0.00) + \
  (abs(eta) <= 2.5) * (pt > 1.0 && pt < 5.0)  * (0.95) +
  (abs(eta) <= 2.5) * (pt > 5.0 && pt < 10.0) * (0.97) +
  (abs(eta) <= 2.5) * (pt > 10.0)             * (0.999) +

  (abs(eta) > 2.5 && abs(eta) <= 4.0) * (pt > 1.0 && pt < 5.0)  * (0.90) +
  (abs(eta) > 2.5 && abs(eta) <= 4.0) * (pt > 5.0 && pt < 10.0) * (0.95) +
  (abs(eta) > 2.5 && abs(eta) <= 4.0) * (pt > 10.0)		* (0.99) +

  (abs(eta) > 4.0 && abs(eta) <= 6.0) * (pt > 1.0 && pt < 5.0)  * (0.85) + \
  (abs(eta) > 4.0 && abs(eta) <= 6.0) * (pt > 5.0 && pt < 10.0) * (0.90) + \
  (abs(eta) > 4.0 && abs(eta) <= 6.0) * (pt > 10.0)             * (0.95) + \
  (abs(eta) > 6.0) * (0.00)
  }

}



################
# Muon isolation
################

# TBC: check values for iso cuts

module Isolation MuonIsolation {
  set CandidateInputArray MuonEfficiency/muons
  set IsolationInputArray EFlowFilter/eflow

  set OutputArray muons

  set DeltaRMax 0.3

  set PTMin 0.5

  set PTRatioMax 0.2
}



###############
# b-tagging L
###############

module BTagging BTaggingL {
  set JetInputArray JetEnergyScale/jets

  set BitNumber 0

  add EfficiencyFormula {5} {

  1.0 * (
  (pt <= 20.0)                                                        * (0.00) +
  (abs(eta) < 4.0)                     * (pt > 20.0 && pt < 50)       * (0.88) +
  (abs(eta) < 4.0)                     * (pt > 50.0 && pt < 100)      * (0.92) +
  (abs(eta) < 4.0)                     * (pt > 100.0 && pt < 200)     * (0.93) +
  (abs(eta) < 4.0)                     * (pt > 200.0 && pt < 500)     * (0.90) +
  (abs(eta) < 4.0)                     * (pt > 500.0 && pt < 1000)    * (0.87) +
  (abs(eta) < 4.0)                     * (pt > 1000.0)                * (0.85) +
  (abs(eta) >= 4.0) * (0.00)
  )

  }

  add EfficiencyFormula {0} {

  (pt <= 20.0)                                                        * (0.00) +
  (abs(eta) < 4.0)                     * (pt > 20.0 && pt < 50)       * (0.15) +
  (abs(eta) < 4.0)                     * (pt > 50.0 && pt < 100)      * (0.08) +
  (abs(eta) < 4.0)                     * (pt > 100.0 && pt < 200)     * (0.06) +
  (abs(eta) < 4.0)                     * (pt > 200.0 && pt < 500)     * (0.08) +
  (abs(eta) < 4.0)                     * (pt > 500.0 && pt < 1000)    * (0.10) +
  (abs(eta) < 4.0)                     * (pt > 1000.0)                * (0.12) +
  (abs(eta) >= 4.0) * (0.00)

  }

  add EfficiencyFormula {4} {

  (pt <= 20.0)                                                        * (0.00) +
  (abs(eta) < 4.0)                     * (pt > 20.0 && pt < 50)       * (0.50) +
  (abs(eta) < 4.0)                     * (pt > 50.0 && pt < 100)      * (0.45) +
  (abs(eta) < 4.0)                     * (pt > 100.0 && pt < 200)     * (0.40) +
  (abs(eta) < 4.0)                     * (pt > 200.0 && pt < 500)     * (0.40) +
  (abs(eta) < 4.0)                     * (pt > 500.0 && pt < 1000)    * (0.40) +
  (abs(eta) < 4.0)                     * (pt > 1000.0)                * (0.40) +
  (abs(eta) >= 4.0) * (0.00)

  }
}

###############
# b-tagging M
###############

module BTagging BTaggingM {
  set JetInputArray JetEnergyScale/jets

  set BitNumber 1

  add EfficiencyFormula {5} {

  1.0 * (
  (pt <= 20.0)                                                        * (0.00) +
  (abs(eta) < 4.0)                     * (pt > 20.0 && pt < 50)       * (0.75) +
  (abs(eta) < 4.0)                     * (pt > 50.0 && pt < 100)      * (0.84) +
  (abs(eta) < 4.0)                     * (pt > 100.0 && pt < 200)     * (0.85) +
  (abs(eta) < 4.0)                     * (pt > 200.0 && pt < 500)     * (0.83) +
  (abs(eta) < 4.0)                     * (pt > 500.0 && pt < 1000)    * (0.80) +
  (abs(eta) < 4.0)                     * (pt > 1000.0)                * (0.80) +
  (abs(eta) >= 4.0) * (0.00)
  )

  }

  add EfficiencyFormula {0} {

  (pt <= 20.0)                                                        * (0.00) +
  (abs(eta) < 4.0)                     * (pt > 20.0 && pt < 50)       * (0.015) +
  (abs(eta) < 4.0)                     * (pt > 50.0 && pt < 100)      * (0.008) +
  (abs(eta) < 4.0)                     * (pt > 100.0 && pt < 200)     * (0.006) +
  (abs(eta) < 4.0)                     * (pt > 200.0 && pt < 500)     * (0.008) +
  (abs(eta) < 4.0)                     * (pt > 500.0 && pt < 1000)    * (0.010) +
  (abs(eta) < 4.0)                     * (pt > 1000.0)                * (0.012) +
  (abs(eta) >= 4.0) * (0.00)

  }

  add EfficiencyFormula {4} {

  (pt <= 20.0)                                                        * (0.00) +
  (abs(eta) < 4.0)                     * (pt > 20.0 && pt < 50)       * (0.20) +
  (abs(eta) < 4.0)                     * (pt > 50.0 && pt < 100)      * (0.15) +
  (abs(eta) < 4.0)                     * (pt > 100.0 && pt < 200)     * (0.13) +
  (abs(eta) < 4.0)                     * (pt > 200.0 && pt < 500)     * (0.12) +
  (abs(eta) < 4.0)                     * (pt > 500.0 && pt < 1000)    * (0.11) +
  (abs(eta) < 4.0)                     * (pt > 1000.0)                * (0.10) +
  (abs(eta) >= 4.0) * (0.00)

  }
}


###############
# b-tagging T
###############

module BTagging BTaggingT {
  set JetInputArray JetEnergyScale/jets

  set BitNumber 2

  add EfficiencyFormula {5} {

  1.0 * (
  (pt <= 20.0)                                                        * (0.00) +
  (abs(eta) < 4.0)                     * (pt > 20.0 && pt < 50)       * (0.55) +
  (abs(eta) < 4.0)                     * (pt > 50.0 && pt < 100)      * (0.65) +
  (abs(eta) < 4.0)                     * (pt > 100.0 && pt < 200)     * (0.70) +
  (abs(eta) < 4.0)                     * (pt > 200.0 && pt < 500)     * (0.65) +
  (abs(eta) < 4.0)                     * (pt > 500.0 && pt < 1000)    * (0.60) +
  (abs(eta) < 4.0)                     * (pt > 1000.0)                * (0.55) +
  (abs(eta) >= 4.0) * (0.00)
  )

  }

  add EfficiencyFormula {0} {

  (pt <= 20.0)                                                        * (0.00) +
  (abs(eta) < 4.0)                     * (pt > 20.0 && pt < 50)       * (0.005) +
  (abs(eta) < 4.0)                     * (pt > 50.0 && pt < 100)      * (0.005) +
  (abs(eta) < 4.0)                     * (pt > 100.0 && pt < 200)     * (0.006) +
  (abs(eta) < 4.0)                     * (pt > 200.0 && pt < 500)     * (0.007) +
  (abs(eta) < 4.0)                     * (pt > 500.0 && pt < 1000)    * (0.007) +
  (abs(eta) < 4.0)                     * (pt > 1000.0)                * (0.008) +
  (abs(eta) >= 4.0) * (0.00)

  }

  add EfficiencyFormula {4} {

  (pt <= 20.0)                                                        * (0.00) +
  (abs(eta) < 4.0)                     * (pt > 20.0 && pt < 50)       * (0.025) +
  (abs(eta) < 4.0)                     * (pt > 50.0 && pt < 100)      * (0.025) +
  (abs(eta) < 4.0)                     * (pt > 100.0 && pt < 200)     * (0.025) +
  (abs(eta) < 4.0)                     * (pt > 200.0 && pt < 500)     * (0.025) +
  (abs(eta) < 4.0)                     * (pt > 500.0 && pt < 1000)    * (0.02) +
  (abs(eta) < 4.0)                     * (pt > 1000.0)                * (0.02) +
  (abs(eta) >= 4.0) * (0.00)

  }
}


###############
# c-tagging L
###############

module BTagging CTaggingL {
  set JetInputArray JetEnergyScale/jets

  set BitNumber 4

  add EfficiencyFormula {4} {

  1.0 * (
  (pt <= 20.0)                                             * (0.00) +
  (abs(eta) < 4.0)                     * (pt > 20.0)       * (0.95)
  )

  }

  add EfficiencyFormula {5} {

  (pt <= 20.0)                                             * (0.00) +
  (abs(eta) < 4.0)                     * (pt > 20.0)       * (0.25)

  }

  add EfficiencyFormula {0} {

  (pt <= 20.0)                                             * (0.00) +
  (abs(eta) < 4.0)                     * (pt > 20.0)       * (0.80)

  }
}


###############
# c-tagging M
###############

module BTagging CTaggingM {
  set JetInputArray JetEnergyScale/jets

  set BitNumber 5

  add EfficiencyFormula {4} {

  1.0 * (
  (pt <= 20.0)                                             * (0.00) +
  (abs(eta) < 4.0)                     * (pt > 20.0)       * (0.65)
  )

  }

  add EfficiencyFormula {5} {

  (pt <= 20.0)                                             * (0.00) +
  (abs(eta) < 4.0)                     * (pt > 20.0)       * (0.07)

  }

  add EfficiencyFormula {0} {

  (pt <= 20.0)                                             * (0.00) +
  (abs(eta) < 4.0)                     * (pt > 20.0)       * (0.14)

  }
}



###############
# c-tagging T
###############

module BTagging CTaggingT {
  set JetInputArray JetEnergyScale/jets

  set BitNumber 6

  add EfficiencyFormula {4} {

  1.0 * (
  (pt <= 20.0)                                             * (0.00) +
  (abs(eta) < 4.0)                     * (pt > 20.0)       * (0.40)
  )

  }

  add EfficiencyFormula {5} {

  (pt <= 20.0)                                             * (0.00) +
  (abs(eta) < 4.0)                     * (pt > 20.0)       * (0.025)

  }

  add EfficiencyFormula {0} {

  (pt <= 20.0)                                             * (0.00) +
  (abs(eta) < 4.0)                     * (pt > 20.0)       * (0.01)

  }
}



#################
# tau-tagging L
#################


module TauTagging TauTaggingL {
  set ParticleInputArray Delphes/allParticles
  set PartonInputArray Delphes/partons
  set JetInputArray JetEnergyScale/jets

  set BitNumber 0

  set DeltaR 0.5

  set TauPTMin 1.0

  set TauEtaMax 4.0

  # add EfficiencyFormula {abs(PDG code)} {efficiency formula as a function of eta and pt}

  add EfficiencyFormula {15} {
  1.0 *
  ((pt <= 20.0)                                                       * (0.00) +
  (abs(eta) < 4.0)                   * (pt > 20.0 && pt <= 100)      * (0.90) +
  (abs(eta) < 4.0)                   * (pt > 100)                    * (0.94) +
  (abs(eta) > 4.0)                   * (0.00)
  )
  }

  add EfficiencyFormula {0} {

  (pt <= 20.0)                                                       * (0.00) +
  (abs(eta) < 4.0)                   * (pt > 20.0 && pt <= 100)      * (0.035) +
  (abs(eta) < 4.0)                   * (pt > 100)                    * (0.02) +
  (abs(eta) > 4.0)                   * (0.00)
  }

}



#################
# tau-tagging M
#################


module TauTagging TauTaggingM {
  set ParticleInputArray Delphes/allParticles
  set PartonInputArray Delphes/partons
  set JetInputArray JetEnergyScale/jets

  set BitNumber 1

  set DeltaR 0.5

  set TauPTMin 1.0

  set TauEtaMax 4.0

  # add EfficiencyFormula {abs(PDG code)} {efficiency formula as a function of eta and pt}

  add EfficiencyFormula {15} {

  1.0 * (
  (pt <= 20.0)                                                       * (0.00) +
  (abs(eta) < 4.0)                   * (pt > 20.0 && pt <= 100)      * (0.80) +
  (abs(eta) < 4.0)                   * (pt > 100)                    * (0.88) +
  (abs(eta) > 4.0)                   * (0.00)
  )
  }

  add EfficiencyFormula {0} {

  (pt <= 20.0)                                                       * (0.00) +
  (abs(eta) < 4.0)                   * (pt > 20.0 && pt <= 100)      * (0.02) +
  (abs(eta) < 4.0)                   * (pt > 100)                    * (0.01) +
  (abs(eta) > 4.0)                   * (0.00)
  }

}


#################
# tau-tagging T
#################


module TauTagging TauTaggingT {
  set ParticleInputArray Delphes/allParticles
  set PartonInputArray Delphes/partons
  set JetInputArray JetEnergyScale/jets

  set BitNumber 2

  set DeltaR 0.5

  set TauPTMin 1.0

  set TauEtaMax 4.0

  # add EfficiencyFormula {abs(PDG code)} {efficiency formula as a function of eta and pt}

  add EfficiencyFormula {15} {

  1.0 * (
  (pt <= 20.0)                                                       * (0.00) +
  (abs(eta) < 4.0)                   * (pt > 20.0 && pt <= 100)      * (0.70) +
  (abs(eta) < 4.0)                   * (pt > 100)                    * (0.82) +
  (abs(eta) > 4.0)                   * (0.00)
  )

  }

  add EfficiencyFormula {0} {

  (pt <= 20.0)                                                       * (0.00) +
  (abs(eta) < 4.0)                   * (pt > 20.0 && pt <= 100)      * (0.01) +
  (abs(eta) < 4.0)                   * (pt > 100)                    * (0.008) +
  (abs(eta) > 4.0)                   * (0.00)
  }

}


#####################################################
# Find uniquely identified photons/electrons/tau/jets
#####################################################

module UniqueObjectFinder UniqueObjectFinder {
# earlier arrays take precedence over later ones
# add InputArray InputArray OutputArray
  add InputArray PhotonIsolation/photons photons
  add InputArray ElectronIsolation/electrons electrons
  add InputArray MuonIsolation/muons muons
  add InputArray JetEnergyScale/jets jets

}


##################
# ROOT tree writer
##################

module TreeWriter TreeWriter {
# add Branch InputArray BranchName BranchClass
  add Branch Delphes/allParticles Particle GenParticle

  add Branch GenMissingET/momentum GenMissingET MissingET

  #add Branch TrackMerger/tracks Track Track
  #add Branch TowerMerger/towers Tower Tower

  #Temporary addition for manual isoVar validation/recalculation
  # add Branch EFlowFilter/eflow ParticleFlowCandidates ParticleFlowCandidate 

  add Branch EFlowTrackMerger/eflowTracks EFlowTrack Track
  add Branch Calorimeter/eflowPhotons EFlowPhoton Tower
  add Branch Calorimeter/eflowNeutralHadrons EFlowNeutralHadron Tower

  add Branch UniqueObjectFinder/photons Photon Photon
  add Branch UniqueObjectFinder/electrons Electron Electron
  add Branch UniqueObjectFinder/muons Muon Muon
  add Branch UniqueObjectFinder/jets Jet Jet 

  #collections for objects before isolation and uniqueobjectfinder
  add Branch PhotonEfficiency/photons PhotonNoIso Photon
  add Branch ElectronEfficiency/electrons ElectronNoIso Electron
  add Branch MuonEfficiency/muons MuonNoIso Muon
  add Branch JetEnergyScale/jets JetNoIso Jet

  #collections for objects after isolation but before uniqueobjectfinder -> not needed?
  # add Branch PhotonIsolation/photons PhotonNoOR Photon
  # add Branch ElectronIsolation/electrons ElectronNoOR Electron
  # add Branch MuonIsolation/muons MuonNoOR Muon

  add Branch GenJetFinder02/jets GenJet02 Jet
  add Branch GenJetFinder04/jets GenJet04 Jet
  add Branch GenJetFinder08/jets GenJet08 Jet
  add Branch GenJetFinder15/jets GenJet15 Jet

  add Branch FastJetFinder02/jets ParticleFlowJet02 Jet
  add Branch FastJetFinder04/jets ParticleFlowJet04 Jet
  add Branch FastJetFinder08/jets ParticleFlowJet08 Jet
  add Branch FastJetFinder15/jets ParticleFlowJet15 Jet

  add Branch CaloJetFinder02/jets CaloJet02 Jet
  add Branch CaloJetFinder04/jets CaloJet04 Jet
  add Branch CaloJetFinder08/jets CaloJet08 Jet
  add Branch CaloJetFinder15/jets CaloJet15 Jet

  add Branch TrackJetFinder02/jets TrackJet02 Jet
  add Branch TrackJetFinder04/jets TrackJet04 Jet
  add Branch TrackJetFinder08/jets TrackJet08 Jet
  add Branch TrackJetFinder15/jets TrackJet15 Jet

  add Branch MissingET/momentum MissingET MissingET
  add Branch ScalarHT/energy ScalarHT ScalarHT
}